/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entity.h"
#include "assert/assert.h"

entitycallback_t ENTITY_CALLBACKS[ENTITY_TYPE_COUNT] = {
  { NULL },
  { playerInit, playerUpdate }
};

entity_t ENTITY_TEST;

void entityInit(
  entity_t *ent,
  const entitytype_t type
) {
  assertNotNull(ent, "Entity cannot be NULL");
  assertTrue(type < ENTITY_TYPE_COUNT, "Invalid entity type");

  // Init values
  memset(ent, 0, sizeof(entity_t));
  ent->type = type;

  assertNotNull(ENTITY_CALLBACKS[type].init, "Entity type init callback err.");
  ENTITY_CALLBACKS[type].init(ent);
}

void entityUpdate(entity_t *ent) {
  assertNotNull(ent, "Entity cannot be NULL");
  assertTrue(ent->type < ENTITY_TYPE_COUNT, "Invalid entity type");

  assertNotNull(ENTITY_CALLBACKS[ent->type].update, "Entity type update err.");
  ENTITY_CALLBACKS[ent->type].update(ent);
}