/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "player.h"

typedef enum {
  ENTITY_TYPE_NULL,
  ENTITY_TYPE_PLAYER,
} entitytype_t;

#define ENTITY_TYPE_COUNT 2

typedef struct {
  void (*init)(entity_t *);
  void (*update)(entity_t *);
} entitycallback_t;

typedef enum {
  FACING_DIRECTION_SOUTH,
  FACING_DIRECTION_EAST,
  FACING_DIRECTION_NORTH,
  FACING_DIRECTION_WEST
} facingdir_t;

typedef struct _entity_t {
  uint8_t x, y;
  uint8_t subX, subY;
  facingdir_t direction;
  entitytype_t type;

  union {
    player_t player;
  };
} entity_t;

/**
 * Initializes an entity.
 * 
 * @param entity The entity to initialize.
 * @param type The type of entity to initialize.
 */
void entityInit(
  entity_t *entity,
  const entitytype_t type
);

/**
 * Updates an entity.
 * 
 * @param entity The entity to update.
 */
void entityUpdate(entity_t *entity);