/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "scene.h"
#include "assert/assert.h"
#include "overworld/overworld.h"

scenetypecallback_t SCENE_CALLBACKS[] = {
  { NULL, NULL, NULL, NULL },// SCENE_TYPE_INITIAL
  { overworldInit, overworldSceneInit, overworldSceneDeinit, overworldUpdate },
};

scene_t SCENE;

void sceneInit() {
  memset(&SCENE, 0, sizeof(scene_t));
  
  for(uint8_t i = 0; i < SCENE_TYPE_COUNT; i++) {
    if(SCENE_CALLBACKS[i].onInit) SCENE_CALLBACKS[i].onInit();
  }

  sceneSet(SCENE_TYPE_OVERWORLD);// Testing
}

void sceneUpdate() {
  assertTrue(SCENE.current < SCENE_TYPE_COUNT, "Invalid Scene Type");

  if(SCENE.next != SCENE.current) {
    SCENE.current = SCENE.next;
    if(SCENE_CALLBACKS[SCENE.current].onActive) {
      SCENE_CALLBACKS[SCENE.current].onActive();
    }
  }

  if(SCENE_CALLBACKS[SCENE.current].onUpdate) {
    SCENE_CALLBACKS[SCENE.current].onUpdate();
  }
}

void sceneSet(const scenetype_t scene) {
  assertTrue(SCENE.next < SCENE_TYPE_COUNT, "Invalid Scene Type");
  SCENE.next = scene;
}